@@ -1,6 +1,6 @@
 ;; examples/nyacc/lang/c99/tryit.scm
 
-;; Copyright (C) 2020-2021 Matthew R. Wette
+;; Copyright (C) 2020-2022 Matthew R. Wette
 ;; 
 ;; Copying and distribution of this file, with or without modification,
 ;; are permitted in any medium without royalty provided the copyright
@@ -626,4 +626,49 @@
     (pp udecl)
     (sf "evaluate:\n")
     (sf "x = ~S\n" (eval-c99-cx expr))))
+
+(when #f
+  (let* ((code
+	  (string-append
+	   "typedef enum let { A = 1, B = 2 } let_t;\n"
+	   "typedef struct { double x; let_t y; } foo_t;\n"
+	   ;;"int res = sizeof(foo_t);\n"
+	   ))
+	 (tree (parse-string code))
+	 (udict (c99-trans-unit->udict tree))
+	 (foo-t '(type-name
+		  (decl-spec-list (type-spec (typename "foo_t")))))
+	 )
+    (pp (find-types foo-t udict '("let_t")))
+    0))
+
+;; bug 57949
+(when #f
+  (let* ((code "
+typedef struct _GObjectClass {
+  void *construct_properties;
+  void *(*constructor)(unsigned long type, unsigned int n_construct_properties
+      , void *construct_properties); } GObjectClass;\n")
+	 (code "struct foo { void bar(void); };\n")
+	 (tree (parse-string code))
+	 (tree (remove-comments tree))
+	 (udict (c99-trans-unit->udict tree))
+	 ;;(udecl (assoc-ref udict "GObjectClass"))
+	 ;;(udecl (assoc-ref udict '(struct . "_GObjectClass")))
+	 (udecl (assoc-ref udict '(struct . "foo")))
+	 (fdecl (fh-cnvt-udecl udecl udict))
+	 (sdecl (with-input-from-string fdecl read))
+	 )
+    ;;(pp udecl)
+    (display fdecl)
+    ;;(pp sdecl)
+    0))
+
+;; bug 62546
+(when #f
+  (let* ((code "#define FOO(/*hello*/X) (X)\nint x = FOO(1);\n")
+	 (tree (parse-string code)))
+    (pp tree)
+    0))
+
 ;; --- last line ---
