@@ -1,6 +1,6 @@
 ;;; lang/c/cpp.scm - C preprocessor
 
-;; Copyright (C) 2015-2019 Matthew R. Wette
+;; Copyright (C) 2015-2022 Matthew R. Wette
 ;;
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -28,6 +28,7 @@
 	    find-incl-in-dirl
 	    eval-cpp-cond-text
 	    expand-cpp-macro-ref
+	    skip-cpp-macro-ref
 	    ;;
 	    expand-cpp-name
 	    parse-cpp-expr
@@ -48,9 +49,8 @@
 (define (sferr fmt . args)
   (apply simple-format (current-error-port) fmt args))
 (use-modules (ice-9 pretty-print))
-(define (pperr exp)
-  (pretty-print exp (current-error-port)))
-
+(define* (pperr exp #:optional (plp ""))
+  (pretty-print exp (current-error-port) #:per-line-prefix plp))
 
 (define c99-std-defs
   '("__DATE__" "__FILE__" "__LINE__" "__STDC__" "__STDC_HOSTED__"
@@ -103,6 +103,13 @@
 	(else (cons ch chl))))
     '() str)))
 
+(define (unesc-c-str str)
+  (list->string
+   (string-fold-right
+    (lambda (ch chl)
+      (if (char=? ch #\\) chl (cons ch chl)))
+    '() str)))
+
 (define ident-like? (make-ident-like-p read-c-ident))
 
 ;; @deffn {Procedure} read-ellipsis ch
@@ -153,6 +160,7 @@
 	(let loop ((args '()) (la (skip-il-ws (read-char))))
 	  (cond
 	   ((eq? la #\)) (reverse args))
+           ((read-c-comm la) (loop args (skip-il-ws (read-char))))
 	   ((read-c-ident la) =>
 	    (lambda (arg) (loop (cons arg args) (skip-il-ws (read-char)))))
 	   ((read-ellipsis la) =>
@@ -286,14 +294,13 @@
 	    ((char) (char->integer (string-ref (tx1 tree) 0)))
 	    ((defined) (if (assoc-ref dict (tx1 tree)) 1 0))
 	    ((has-include)
-	     (if (find-incl-in-dirl (tx1 tree) inc-dirs #f) 1 0))
+	     (if (find-incl-in-dirl (unesc-c-str (tx1 tree)) inc-dirs #f) 1 0))
 	    ((has-include-next)
-	     (if (find-incl-in-dirl (tx1 tree) inc-dirs #t) 1 0))
+	     (if (find-incl-in-dirl (unesc-c-str (tx1 tree)) inc-dirs #t) 1 0))
 	    ((pre-inc post-inc) (1+ (ev1 tree)))
 	    ((pre-dec post-dec) (1- (ev1 tree)))
 	    ((pos) (ev1 tree))
 	    ((neg) (- (ev1 tree)))
-
 	    ((not) (if (zero? (ev1 tree)) 1 0))
 	    ((mul) (* (ev1 tree) (ev2 tree)))
 	    ((div) (/ (ev1 tree) (ev2 tree)))
@@ -400,7 +407,7 @@
   (let ((ch (read-char)))
     ;; if exit, then did not defined __has_include(X)=__has_include__(X)
     (if (or (eof-object? ch) (not (char=? #\( ch)))
-	(throw 'cpp-error "expedcting `('")))
+	(throw 'cpp-error "expecting `('")))
   (let loop ((chl '()) (ch (skip-il-ws (read-char))))
     (cond
      ((eof-object? ch) (cpp-err "illegal argument"))
@@ -432,7 +439,7 @@
       (if (pair? rr)
 	  (cpp-expand-text repl defs (append rr used)) ;; re-run
 	  repl)))
-     
+
   (let loop ((rr '())			; list symbols resolved
 	     (tkl '())			; token list of
 	     (lv 0)			; level
@@ -652,6 +659,22 @@
      ;;^ does not work here: move here when cpp is token-based
      (else #f))))
 
+;; may not catch strings w/ non-matching parens
+(define (skip-cpp-macro-ref name defs)
+  (let ((rval (assoc-ref defs name)))
+    (and
+     (pair? rval)
+     (let loop ((ch (read-char)))
+       (cond ((eof-object? ch) (throw 'c99-error "eof when expecting `('"))
+	     ((char-whitespace? ch) (loop (read-char)))
+	     ((char=? ch #\() #t)
+	     (else (unread-char ch) #f)))
+     (let loop ((lv 0) (ch (read-char)))
+       (cond ((eof-object? ch) (throw 'c99-error "expecting `)'"))
+	     ((char=? #\( ch) (loop (1+ lv) (read-char)))
+	     ((char=? #\) ch) (if (zero? lv) #f (loop (1- lv) (read-char))))
+	     (else (loop lv (read-char))))))))
+
 ;; @deffn {Procedure} expand-cpp-name name defs => repl|#f
 ;; Calls @code{expand-cpp-macro-ref} with null input string (w/o further
 ;; input).  If @var{name} is has a function definition @code{#f} is returned.
