@@ -1,6 +1,6 @@
 ;;; examples/nyacc/lang/c99/ffi-help.scm
 
-;; Copyright (C) 2016-2021 Matthew R. Wette
+;; Copyright (C) 2016-2022 Matthew R. Wette
 ;;
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -94,7 +94,7 @@
   #:use-module (ice-9 regex)
   #:use-module (ice-9 pretty-print)
   #:re-export (*nyacc-version*)
-  #:version (1 06 8))
+  #:version (1 07 0))
 
 (define fh-cpp-defs
   (cond
@@ -114,6 +114,7 @@
 (set! fh-inc-dirs (cons "." fh-inc-dirs))
 
 ;; maybe change to a record-type
+(define *BS-version* 1)
 (define *options* (make-parameter '()))
 (define *debug-parse* (make-parameter #f)) ; parse debug mode
 (define *show-incs* (make-parameter #f))   ; show include directories
@@ -173,6 +174,9 @@
 
 ;; === utilities
 
+(define (sfstr fmt . args)
+  (apply simple-format #f fmt args))
+
 (define (make-arg-namer)
   (let ((ix 0))
     (lambda ()
@@ -335,14 +339,15 @@
 	  (list ,@(map (lambda (l) `(dynamic-link ,l)) (reverse libraries))))))
     (sfscm "\n")
     (if (*echo-decls*) (sfscm "(define echo-decls #t)\n\n"))
-    ;;
-    (ppscm
+    ;; moved to (system ffi-help-rt)
+    #;(ppscm
      '(cond-expand
        (guile-2.2)
        (guile-2
 	(define intptr_t long)
-	(define uintptr_t unsigned-long))))
-    (sfscm "\n")))
+	(define uintptr_t unsigned-long))
+       (guile)))
+    #;(sfscm "\n")))
 
 ;; === type conversion ==============
 
@@ -367,13 +372,13 @@
     ("signed short int" . short) ("int" . int) ("signed" . int)
     ("signed int" . int) ("long" . long) ("long int" . long)
     ("signed long" . long) ("signed long int" . long) ("long long" . long)
-    ("long long int" . long) ("signed long long" . long)
+    ("long long int" . long-long) ("signed long long" . long-long)
     ("signed long long int" . long)
     ("unsigned short int" . unsigned-short) ("unsigned short" . unsigned-short)
     ("unsigned int" . unsigned-int) ("unsigned" . unsigned-int)
     ("unsigned long int" . unsigned-long) ("unsigned long" . unsigned-long)
-    ("unsigned long long int" . unsigned-long)
-    ("unsigned long long" . unsigned-long)
+    ("unsigned long long int" . unsigned-long-long)
+    ("unsigned long long" . unsigned-long-long)
     ("intptr_t" . intptr_t) ("uintptr_t" . uintptr_t) ("size_t" . size_t)
     ("ssize_t" . ssize_t) ("ptrdiff_t" . ptrdiff_t)
     ("int8_t" . int8) ("uint8_t" . uint8) 
@@ -453,11 +458,13 @@
       (((pointer-to) (float-type ,fx-name))
        `(fh:pointer ,(assoc-ref bs-typemap fx-name)))
 
-      ;; bs does not support function pointers
-      (((function-returning . ,rest) . ,rest)
-       `(fh:pointer 'void))
-      (((pointer-to) (function-returning . ,rest) . ,rest)
-       `(fh:pointer 'void))
+      ;; bs does not support function pointers, but fh does now
+      (((function-returning (param-list . ,params)) . ,tail)
+       `(fh:function ,(mtail->bs-desc tail)
+		     (list ,@(gen-bs-decl-params params))))
+      (((pointer-to) (function-returning (param-list . ,params)) . ,tail)
+       `(fh:function ,(mtail->bs-desc tail)
+		     (list ,@(gen-bs-decl-params params))))
       (((pointer-to) (pointer-to) (function-returning . ,rest) . ,rest)
        `(fh:pointer 'void))
 
@@ -592,24 +599,39 @@
 	     (cons (expand-typerefs pair udict defined) seed))
 	   '() (clean-and-unitize-fields (cdr field-list))))))
 
+;; Convert field list for bytestructures
 ;; field-list is (field-list . ,fields)
-;;(define (cnvt-field-list field-list)
 (define-public (cnvt-field-list field-list)
 
+  (define (x-acons-defn name type seed)
+    (cons (eval-string (sfstr "(quote `(~A ,~S))" name type)) seed))
+
   (define (acons-defn name type seed)
-    (cons (eval-string (simple-format #f "(quote `(~A ,~S))" name type)) seed))
+    (cons
+     (eval-string
+      (if (string=? name "@")           ; dirty sol'n for BS v2 anon unions
+          (sfstr "(quote `(~S ,~S))"
+                 (case (car type) ((bs:struct) 'struct) ((bs:union) 'union))
+                 (cadr type))
+          (sfstr "(quote `(~A ,~S))" name type)))
+     seed))
 
   (define (acons-bfld name type seed)	; bit-field
     (let ((size (list-ref type 1)) (type (list-ref type 2)))
-      (cons (eval-string
-	     (simple-format #f "(quote `(~A ,~S ~A))" name type size)) seed)))
+      (cons (eval-string (sfstr "(quote `(~A ,~S ~A))" name type size)) seed)))
 
-  (define anon-namer
+  (define anon-namer-1                  ; for BS v1 (a workaround)
     (let ((ix 0))
       (lambda ()
 	(set! ix (1+ ix))
 	(string-append "_" (number->string ix)))))
 
+  (define anon-namer-2                  ; for BS v2 (see BS issue #43)
+    (lambda () "@"))
+
+  (define anon-namer
+    (case *BS-version* ((1) anon-namer-1) ((2) anon-namer-2)))
+
   (define (unitize decl seed)
     (dictize-comp-decl decl seed #:namer anon-namer))
 
@@ -731,9 +753,52 @@
 
 ;; === function declarations : signatures for pointer->procedure
 
+(define fh-cpp-dict
+  (map (lambda (ent)
+	 (let ((elts (string-split ent #\=)))
+	   (cons (car elts) (if (null? (cdr elts)) "" (cadr elts)))))
+       fh-cpp-defs))
+
+(define ffi-long-long #f)
+(define ffi-unsigned-long-long #f)
+
+;;(pperr fh-cpp-dict) 
+(case (and=> (assoc-ref fh-cpp-dict "__LONG_LONG_WIDTH__") string->number)
+  ((64)
+   (set! ffi-long-long 'ffi:int64)
+   (set! ffi-unsigned-long-long 'ffi:uint64))
+  ((32)
+   (set! ffi-long-long 'ffi:int32)
+   (set! ffi-unsigned-long-long 'ffi:uint32))
+  (else
+   (sferr "ffi-help: warning: unknown ffi type: long-long")
+   (set! ffi-long-long 'ffi:long)
+   (set! ffi-unsigned-long-long 'ffi:unsigned-long)))
+
+(define ffi-intptr_t #f)
+(define ffi-uintptr_t #f)
+
+(cond-expand
+ ((or guile-3 guile-2.2)
+  (set! ffi-intptr_t 'ffi:intptr_t)
+  (set! ffi-uintptr_t 'ffi:uintptr_t))
+ (guile-2
+  (case (and=> (assoc-ref fh-cpp-dict "__INTPTR_WIDTH__") string->number)
+    ((64)
+     (set! ffi-intptr_t 'ffi:int64_t)
+     (set! ffi-uintptr_t 'ffi:uint64_t))
+    ((32)
+     (set! ffi-intptr_t 'ffi:int32_t)
+     (set! ffi-uintptr_t 'ffi:uint32_t))
+    (else
+     (sferr "ffi-help: warning: unknown ffi type: intptr_t\n")
+     (set! ffi-intptr_t 'ffi:long)
+     (set! ffi-uintptr_t 'ffi:unsigned-long))))
+ (guile))
+ 
 (define ffi-typemap
   ;; see system/foreign.scm
-  '(("void" . ffi:void) ("float" . ffi:float) ("double" . ffi:double)
+  `(("void" . ffi:void) ("float" . ffi:float) ("double" . ffi:double)
     ;;
     ("short" . ffi:short) ("short int" . ffi:short) ("signed short" . ffi:short)
     ("signed short int" . ffi:short) ("int" . ffi:int) ("signed" . ffi:int)
@@ -752,13 +817,14 @@
     ("int32_t" . ffi:int32) ("uint32_t" . ffi:uint32) 
     ("int64_t" . ffi:int64) ("uint64_t" . ffi:uint64)
     ;; hacks
-    ("intptr_t" . ffi:long) ("uintptr_t" . ffi:unsigned-long)
+    ("intptr_t" . ,ffi-intptr_t) ("uintptr_t" . ,ffi-uintptr_t)
     ("char" . ffi:int8) ("signed char" . ffi:int8) ("unsigned char" . ffi:uint8)
     ("wchar_t" . int) ("char16_t" . int16) ("char32_t" . int32)
-    ("long long" . ffi:long) ("long long int" . ffi:long)
-    ("signed long long" . ffi:long) ("signed long long int" . ffi:long)
-    ("unsigned long long" . ffi:unsigned-long)
-    ("unsigned long long int" . ffi:unsigned-long)
+    ("long long" . ,ffi-long-long) ("long long int" . ,ffi-long-long)
+    ("signed long long" . ,ffi-long-long)
+    ("signed long long int" . ,ffi-long-long)
+    ("unsigned long long" . ,ffi-unsigned-long-long)
+    ("unsigned long long int" . ,ffi-unsigned-long-long)
     ("_Bool" . ffi:int8)))
 
 (define ffi-defined (map car ffi-typemap))
@@ -1803,18 +1869,11 @@
 	(decl-spec-list
 	 (stor-spec (typedef))
 	 (type-spec (fixed-type "unsigned char")))
-	(init-declr
-	 (array-of
-	  (ident "hdset_reg_ref_t")
-	  (add (sizeof-type
-		(type-name
-		 (decl-spec-list (type-spec (typename "haddr_t")))))
-	       (p-expr (fixed "4"))))))
-       (let* ((typename "haddr_t")
-	      (size (+ (sizeof '*) 4)))
-	 (sfscm "(define-public ~A-desc (bs:vector ~A '*))\n" typename size)
-	 (fhscm-def-compound typename)
-	 (values (cons typename wrapped) (cons typename defined))))
+	(init-declr (ary-declr (ident ,typename) ,dim)))
+       (let* ((sz (eval-c99-cx dim (*udict*))))
+	(sfscm "(define-public ~A-desc (bs:vector ~A int8))\n" typename sz)
+	(fhscm-def-compound typename)
+	(values (cons typename wrapped) (cons typename defined))))
 
       ;; from gtk+-3.0/gtk/gtk.h
       ((udecl (decl-spec-list
